/*
 * IRC - Internet Relay Chat, ircd/m_webirc.c
 * Copyright (C) 1990 Jarkko Oikarinen and
 *                    University of Oulu, Computing Center
 *
 * See file AUTHORS in IRC package for additional names of
 * the programmers.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: m_tmpl.c 1271 2004-12-11 05:14:07Z klmitch $
 */

/*
 * m_functions execute protocol messages on this server:
 *
 *    cptr    is always NON-NULL, pointing to a *LOCAL* client
 *            structure (with an open socket connected!). This
 *            identifies the physical socket where the message
 *            originated (or which caused the m_function to be
 *            executed--some m_functions may call others...).
 *
 *    sptr    is the source of the message, defined by the
 *            prefix part of the message if present. If not
 *            or prefix not found, then sptr==cptr.
 *
 *            (!IsServer(cptr)) => (cptr == sptr), because
 *            prefixes are taken *only* from servers...
 *
 *            (IsServer(cptr))
 *                    (sptr == cptr) => the message didn't
 *                    have the prefix.
 *
 *                    (sptr != cptr && IsServer(sptr) means
 *                    the prefix specified servername. (?)
 *
 *                    (sptr != cptr && !IsServer(sptr) means
 *                    that message originated from a remote
 *                    user (not local).
 *
 *            combining
 *
 *            (!IsServer(sptr)) means that, sptr can safely
 *            taken as defining the target structure of the
 *            message in this server.
 *
 *    *Always* true (if 'parse' and others are working correct):
 *
 *    1)      sptr->from == cptr  (note: cptr->from == cptr)
 *
 *    2)      MyConnect(sptr) <=> sptr == cptr (e.g. sptr
 *            *cannot* be a local connection, unless it's
 *            actually cptr!). [MyConnect(x) should probably
 *            be defined as (x == x->from) --msa ]
 *
 *    parc    number of variable parameter strings (if zero,
 *            parv is allowed to be NULL)
 *
 *    parv    a NULL terminated list of parameter pointers,
 *
 *                    parv[0], sender (prefix string), if not present
 *                            this points to an empty string.
 *                    parv[1]...parv[parc-1]
 *                            pointers to additional parameters
 *                    parv[parc] == NULL, *always*
 *
 *            note:   it is guaranteed that parv[0]..parv[parc-1] are all
 *                    non-NULL pointers.
 */
#include "config.h"

#include "client.h"
#include "hash.h"
#include "ircd.h"
#include "ircd_features.h"
#include "ircd_log.h"
#include "ircd_reply.h"
#include "ircd_string.h"
#include "numeric.h"
#include "numnicks.h"
#include "send.h"
#include "s_conf.h"
#include "s_misc.h"
#include "querycmds.h"

/* #include <assert.h> -- Now using assert in ircd_log.h */

/*
 * m_webirc
 *
 * parv[0] = sender prefix
 * parv[1] = password           (WEBIRC Password)
 * parv[2] = username           (Username from WEBIRC)
 * parv[3] = hostname           (Real host)
 * parv[4] = ip                 (Real IP in ASCII)
 */
int m_webirc(struct Client* cptr, struct Client* sptr, int parc, char* parv[])
{
  struct irc_in_addr addr;
  char* username = NULL;
  char* hostname = NULL;
  char* ipaddr = NULL;
  char* password = NULL;
  int res = 0;
  int killreason;

  if (IsServerPort(cptr))
    return exit_client(cptr, sptr, &me, "Use a different port");

  if (parc < 5)
    return need_more_params(sptr, "WEBIRC");

  /* These shouldn't be empty, but just in case... */
  if (!EmptyString(parv[1]))
    password = parv[1];
  if (!EmptyString(parv[2]))
    username = parv[2];
  if (!EmptyString(parv[3]))
    hostname = parv[3];
  if (!EmptyString(parv[4]))
    ipaddr = parv[4];

  /* And to be extra sure... */
  if (!password || !hostname || !ipaddr)
    return exit_client(cptr, sptr, &me, "WEBIRC parameters supplied are invalid");

  find_webirc_conf(cptr, password, &res);
  switch (res)
  {
    case 2:
      return exit_client(cptr, sptr, &me, "WEBIRC Not authorized from your host");
      break;
    case 1:
      return exit_client(cptr, sptr, &me, "WEBIRC Password invalid for your host");
      break;
  }

  /* Send connection notice to inform opers of the change of IP and host. */
  if (feature_bool(FEAT_CONNEXIT_NOTICES))
    sendto_opmask_butone(0, SNO_CONNEXIT,
                         "WEBIRC Client host: from %s [%s] to %s [%s]",
                         cli_sockhost(sptr), cli_sock_ip(sptr), hostname, ipaddr);

  /* Undo original IP connection in IPcheck. */
  IPcheck_connect_fail(sptr);
  /* Need to disconnect original IP otherwise the count will never go down. */
  if (IsIPChecked(sptr))
    IPcheck_disconnect(sptr);
  ClearIPChecked(sptr);

  /* Update the IP and charge them as a remote connect. */
  ircd_aton(&addr, ipaddr);
  memcpy(&cli_ip(sptr), &addr, sizeof(cli_ip(sptr)));
  IPcheck_remote_connect(sptr);

  /* Change cli_sock_ip() and cli_sockhost() to spoofed host and IP. */
  ircd_strncpy(cli_sock_ip(sptr), ipaddr, SOCKIPLEN);
  ircd_strncpy(cli_sockhost(sptr), hostname, HOSTLEN);
  ircd_strncpy(cli_username(sptr), username, 8);

  /* Update host names if already set. */
  if (cli_user(sptr)) {
    if (!HasHiddenHost(sptr))
      ircd_strncpy(cli_user(sptr)->host, hostname, HOSTLEN);
    ircd_strncpy(cli_user(sptr)->realhost, hostname, HOSTLEN);
  }

  /* Check for K- or G-line. */
  killreason = find_kill(sptr);
  if (killreason) {
    ServerStats->is_ref++;
    return exit_client(sptr, sptr, &me,
                       (killreason == -1 ? "K-lined" : "G-lined"));
  }

  SetWebirc(sptr);

/* 
  ++UserStats.webirc;
*/

  return 0;
}

